# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine DownloadableACL API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from builtins import *
from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class DownloadableAcl(object):
    """Identity Services Engine DownloadableACL API (version: 3.3_patch_1).

    Wraps the Identity Services Engine DownloadableACL
    API and exposes the API as native Python
    methods that return native Python objects.

    | Downloadable ACL API allows the client to add, delete, update, search and perform actions on downloadable ACL.

    **Revision History**

    +------------+--------------------+-------------------+-----------------------------------------+-----------------------+-----------------------------------+
    | Revision # | Resource   Version | Cisco ISE Version | Description                             | Revision Modification | Revision Modification             |
    +------------+--------------------+-------------------+-----------------------------------------+-----------------------+-----------------------------------+
    |            |                    |                   |                                         | Attribute             | Description                       |
    +------------+--------------------+-------------------+-----------------------------------------+-----------------------+-----------------------------------+
    | 0          | 1.0                | 2.3               | Initial Cisco ISE Version               |                       |                                   |
    +------------+--------------------+-------------------+-----------------------------------------+-----------------------+-----------------------------------+
    | 1          | 1.1                | 2.6               | Support new attribute IP version in 2.6 | daclType              | Added Enum   Attribute 'daclType' |
    +------------+--------------------+-------------------+-----------------------------------------+-----------------------+-----------------------------------+

    |

    **Resource Definition**

    +---------------+-----------+--------------+---------------------------------------------------------------------------------------+--------------------+--------------------------------------+
    | **Attribute** | **Type**  | **Required** | **Description**                                                                       | **Default Values** | **Example Values**                   |
    +---------------+-----------+--------------+---------------------------------------------------------------------------------------+--------------------+--------------------------------------+
    | name          | String    | Yes          | Resource Name. Name may contain alphanumeric or any of the following characters [_.-] |                    | multiline_acl                        |
    +---------------+-----------+--------------+---------------------------------------------------------------------------------------+--------------------+--------------------------------------+
    | id            | String    | Yes          | Resource UUID                                                                         |                    | c1e0e9e0-717f-11eb-9fb2-b6cb23d38630 |
    +---------------+-----------+--------------+---------------------------------------------------------------------------------------+--------------------+--------------------------------------+
    | description   | String    | No           | Use the string \\n for a newline                                                       |                    | description                          |
    +---------------+-----------+--------------+---------------------------------------------------------------------------------------+--------------------+--------------------------------------+
    | dacl          | String    | Yes          | The DACL Content. Use the string \\n for a newline                                     |                    | permit ip any any                    |
    +---------------+-----------+--------------+---------------------------------------------------------------------------------------+--------------------+--------------------------------------+
    | daclType      | Enum      | No           | Allowed values: IPV4, IPV6, IP_AGNOSTIC                                               | IPV4               |                                      |
    +---------------+-----------+--------------+---------------------------------------------------------------------------------------+--------------------+--------------------------------------+

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new DownloadableAcl
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(DownloadableAcl, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_downloadable_acl_by_id(self,
                                   id,
                                   headers=None,
                                   **query_parameters):
        """This API allows the client to get a downloadable ACL by ID.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/downloadableacl/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_dfa8f48210e85715beebb44e62fac408_v3_3_patch_1', _api_response)

    def get_by_id(self,
                  id,
                  headers=None,
                  **query_parameters):
        """Alias for `get_downloadable_acl_by_id <#ciscoisesdk.
        api.v3_3_patch_1.downloadable_acl.
        DownloadableAcl.get_downloadable_acl_by_id>`_
        """
        return self.get_downloadable_acl_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def update_downloadable_acl_by_id(self,
                                      id,
                                      dacl=None,
                                      dacl_type=None,
                                      description=None,
                                      name=None,
                                      headers=None,
                                      payload=None,
                                      active_validation=True,
                                      **query_parameters):
        """This API allows the client to update a downloadable ACL.

        Args:
            dacl(string): The DACL Content. Use the string \\n for a
                newline, property of the request body.
            dacl_type(string): Allowed values: IPV4, IPV6,
                IP_AGNOSTIC, property of the request
                body.
            description(string): Use the string \\n for a newline,
                property of the request body.
            id(string): id, property of the request body.
            name(string): Resource Name. Name may contain
                alphanumeric or any of the following
                characters [_.-], property of the
                request body.
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'id':
                    id,
                'name':
                    name,
                'description':
                    description,
                'dacl':
                    dacl,
                'daclType':
                    dacl_type,
            }
            _payload = {
                'DownloadableAcl': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_d8c7ba0cb8f56d99135e16d2d973d11_v3_3_patch_1')\
                .validate(_payload)

        e_url = ('/ers/config/downloadableacl/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_d8c7ba0cb8f56d99135e16d2d973d11_v3_3_patch_1', _api_response)

    def update_by_id(self,
                     id,
                     dacl=None,
                     dacl_type=None,
                     description=None,
                     name=None,
                     headers=None,
                     payload=None,
                     active_validation=True,
                     **query_parameters):
        """Alias for `update_downloadable_acl_by_id <#ciscoisesdk.
        api.v3_3_patch_1.downloadable_acl.
        DownloadableAcl.update_downloadable_acl_by_id>`_
        """
        return self.update_downloadable_acl_by_id(
            id=id,
            dacl=dacl,
            dacl_type=dacl_type,
            description=description,
            name=name,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def delete_downloadable_acl_by_id(self,
                                      id,
                                      headers=None,
                                      **query_parameters):
        """This API deletes a downloadable ACL.

        Args:
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        e_url = ('/ers/config/downloadableacl/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b3db444eaa50678218c29f88de60e8_v3_3_patch_1', _api_response)

    def delete_by_id(self,
                     id,
                     headers=None,
                     **query_parameters):
        """Alias for `delete_downloadable_acl_by_id <#ciscoisesdk.
        api.v3_3_patch_1.downloadable_acl.
        DownloadableAcl.delete_downloadable_acl_by_id>`_
        """
        return self.delete_downloadable_acl_by_id(
            id=id,
            headers=headers,
            **query_parameters
        )

    def get_downloadable_acl(self,
                             page=None,
                             size=None,
                             headers=None,
                             **query_parameters):
        """This API allows the client to get all downloadable ACLs.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))

        _params = {
            'page':
                page,
            'size':
                size,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/downloadableacl')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_bc200af85d598885a990ff9bcbf8_v3_3_patch_1', _api_response)

    def get_all(self,
                page=None,
                size=None,
                headers=None,
                **query_parameters):
        """Alias for `get_downloadable_acl <#ciscoisesdk.
        api.v3_3_patch_1.downloadable_acl.
        DownloadableAcl.get_downloadable_acl>`_
        """
        return self.get_downloadable_acl(
            page=page,
            size=size,
            headers=headers,
            **query_parameters
        )

    def get_downloadable_acl_generator(self,
                                       page=None,
                                       size=None,
                                       headers=None,
                                       **query_parameters):
        """This API allows the client to get all downloadable ACLs.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_downloadable_acl, dict(
                page=page,
                size=size,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def get_all_generator(self,
                          page=None,
                          size=None,
                          headers=None,
                          **query_parameters):
        """Alias for `get_downloadable_acl_generator <#ciscoisesdk.
        api.v3_3_patch_1.downloadable_acl.
        DownloadableAcl.get_downloadable_acl_generator>`_
        """
        yield from get_next_page(
            self.get_downloadable_acl, dict(
                page=page,
                size=size,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["SearchResult", "nextPage", "href"],
            access_resource_list=["SearchResult", "resources"])

    def create_downloadable_acl(self,
                                dacl=None,
                                dacl_type=None,
                                description=None,
                                name=None,
                                headers=None,
                                payload=None,
                                active_validation=True,
                                **query_parameters):
        """This API creates a downloadable ACL.

        Args:
            dacl(string): The DACL Content. Use the string \\n for a
                newline, property of the request body.
            dacl_type(string): Allowed values: IPV4, IPV6,
                IP_AGNOSTIC, property of the request
                body.
            description(string): Use the string \\n for a newline,
                property of the request body.
            name(string): Resource Name. Name may contain
                alphanumeric or any of the following
                characters [_.-], property of the
                request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'name':
                    name,
                'description':
                    description,
                'dacl':
                    dacl,
                'daclType':
                    dacl_type,
            }
            _payload = {
                'DownloadableAcl': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_adcf947c42fe5588b7b82d9c43a3bbf0_v3_3_patch_1')\
                .validate(_payload)

        e_url = ('/ers/config/downloadableacl')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_adcf947c42fe5588b7b82d9c43a3bbf0_v3_3_patch_1', _api_response)

    def create(self,
               dacl=None,
               dacl_type=None,
               description=None,
               name=None,
               headers=None,
               payload=None,
               active_validation=True,
               **query_parameters):
        """Alias for `create_downloadable_acl <#ciscoisesdk.
        api.v3_3_patch_1.downloadable_acl.
        DownloadableAcl.create_downloadable_acl>`_
        """
        return self.create_downloadable_acl(
            dacl=dacl,
            dacl_type=dacl_type,
            description=description,
            name=name,
            payload=payload,
            active_validation=active_validation,
            headers=headers,
            **query_parameters
        )

    def get_version(self,
                    headers=None,
                    **query_parameters):
        """This API helps to retrieve the version information related to
        the downloadable ACL.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/downloadableacl/versioninfo')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d10b7914625e5da0861cbeab4cf6440e_v3_3_patch_1', _api_response)

    def patch_downloadable_acl_id(self,
                                  id,
                                  dacl=None,
                                  dacl_type=None,
                                  description=None,
                                  name=None,
                                  headers=None,
                                  payload=None,
                                  active_validation=True,
                                  **query_parameters):
        """Update any attribute subset. Only attributes that sent will be
        affected.

        Args:
            dacl(string): The DACL Content, property of the request
                body.
            dacl_type(string): Allowed values: IPV4, IPV6,
                IP_AGNOSTIC, property of the request
                body.
            description(string): Description, property of the
                request body.
            id(string): Id, property of the request body.
            name(string): name, property of the request body.
            id(str): id path parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(list): A list of MyDict objects. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _tmp_payload = {
                'dacl':
                    dacl,
                'daclType':
                    dacl_type,
                'name':
                    name,
                'id':
                    id,
                'description':
                    description,
            }
            _payload = {
                'DownloadableAcl': dict_from_items_with_values(_tmp_payload)
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b0514ee0516d57978711c201ba54dcd9_v3_3_patch_1')\
                .validate(_payload)

        e_url = ('/ers/config/downloadableacl/{id}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.patch(endpoint_full_url, params=_params,
                                                headers=_headers,
                                                **request_params)
        else:
            _api_response = self._session.patch(endpoint_full_url, params=_params,
                                                **request_params)

        return self._object_factory('bpm_b0514ee0516d57978711c201ba54dcd9_v3_3_patch_1', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine ACIBindings API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from builtins import *
from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class AciBindings(object):
    """Identity Services Engine ACIBindings API (version: 3.1.1).

    Wraps the Identity Services Engine ACIBindings
    API and exposes the API as native Python
    methods that return native Python objects.

    | The ACI Bindings API allows clients to retrieve bindings that were sent to Cisco ISE by ACI or received on ACI from Cisco ISE. The binding information will be identical to the information on the ACI bindings page in the Cisco ISE UI. Filtering will be based on one attribute only, such as ip, sgt, vn, psn, learnedFrom, or learnedBy with CONTAINS mode of search.

    **Revision History**

    +----------------+----------------------+-----------------------+---------------------------+
    | **Revision #** | **Resource Version** | **Cisco ISE Version** | **Description**           |
    +----------------+----------------------+-----------------------+---------------------------+
    | 0              | 1.0                  | 3.0                   | Initial Cisco ISE Version |
    +----------------+----------------------+-----------------------+---------------------------+

    |

    **Resource Definition**

    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | **Attribute** | **Type**    | **Required** | **Description**                                                                                         | **Example Values**                   |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | name          | String      | Yes          | Resource Name                                                                                           |                                      |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | id            | String      | No           | Resource UUID value                                                                                     | f9269682-dcaf-11e3-ad0a-5bdcd2d9fd69 |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | description   | String      | No           |                                                                                                         |                                      |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | ip            | String      | Yes          | Binding IPv4 address. Each binding will be exclusively identified by its IP address and virtual network | 10.0.0.1                             |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | sgt           | String      | No           | Security Group Tag (SGT) value. The valid range for SGT values is 0-65534                               | 1234                                 |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | vn            | String      | Yes          | Virtual network. Each binding will be exclusively identified by its IP address and virtual network      | vn1234                               |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | psn           | String      | No           | Cisco ISE Policy Service node (PSN) IP address                                                          | 10.86.189.216                        |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | learnedFrom   | String      | Yes          | Binding Source                                                                                          | ISE by ACI                           |
    |               |             |              |                                                                                                         | ACI by ISE                           |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+
    | learnedBy     | String      | Yes          | Binding Type                                                                                            | SXP(SXP, 0)                          |
    |               |             |              |                                                                                                         | STATIC(Static, 1)                    |
    |               |             |              |                                                                                                         | RADIUS(RADIUS, 2)                    |
    |               |             |              |                                                                                                         | ACI(ACI, 3)                          |
    +---------------+-------------+--------------+---------------------------------------------------------------------------------------------------------+--------------------------------------+

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new AciBindings
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(AciBindings, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_aci_bindings(self,
                         filter_by=None,
                         filter_value=None,
                         page=None,
                         size=None,
                         sort=None,
                         sort_by=None,
                         headers=None,
                         **query_parameters):
        """This API allows clients to retrieve all the bindings that were
        sent to Cisco ISE by ACI or received on ACI from Cisco
        ISE.The binding information will be identical to the
        information on ACI bindings page in the Cisco ISE UI.
        Filtering will be based on one attribute only, such as
        ip/sgt/vn/psn/learnedFrom/learnedBy with CONTAINS mode
        of search.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            sort(str): sort query parameter. sort type asc or
                desc.
            sort_by(str): sortBy query parameter. sort column
                by which objects needs to be sorted.
            filter_by(str, list, set, tuple): filterBy query
                parameter.
            filter_value(str, list, set, tuple): filterValue
                query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)
            if 'ERS-Media-Type' in headers:
                check_type(headers.get('ERS-Media-Type'),
                           str)
            if 'X-CSRF-Token' in headers:
                check_type(headers.get('X-CSRF-Token'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))
        check_type(sort, str)
        check_type(sort_by, str)
        check_type(filter_by, (str, list, set, tuple))
        check_type(filter_value, (str, list, set, tuple))

        _params = {
            'page':
                page,
            'size':
                size,
            'sort':
                sort,
            'sortBy':
                sort_by,
            'filterBy':
                filter_by,
            'filterValue':
                filter_value,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/acibindings/getall')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d1448851f0154d0b6e9c856ec6cc6f0_v3_1_1', _api_response)

    def get_all(self,
                filter_by=None,
                filter_value=None,
                page=None,
                size=None,
                sort=None,
                sort_by=None,
                headers=None,
                **query_parameters):
        """Alias for `get_aci_bindings <#ciscoisesdk.
        api.v3_1_1.aci_bindings.
        AciBindings.get_aci_bindings>`_
        """
        return self.get_aci_bindings(
            filter_by=filter_by,
            filter_value=filter_value,
            page=page,
            size=size,
            sort=sort,
            sort_by=sort_by,
            headers=headers,
            **query_parameters
        )

    def get_aci_bindings_generator(self,
                                   filter_by=None,
                                   filter_value=None,
                                   page=None,
                                   size=None,
                                   sort=None,
                                   sort_by=None,
                                   headers=None,
                                   **query_parameters):
        """This API allows clients to retrieve all the bindings that were
        sent to Cisco ISE by ACI or received on ACI from Cisco
        ISE.The binding information will be identical to the
        information on ACI bindings page in the Cisco ISE UI.
        Filtering will be based on one attribute only, such as
        ip/sgt/vn/psn/learnedFrom/learnedBy with CONTAINS mode
        of search.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            sort(str): sort query parameter. sort type asc or
                desc.
            sort_by(str): sortBy query parameter. sort column
                by which objects needs to be sorted.
            filter_by(str, list, set, tuple): filterBy query
                parameter.
            filter_value(str, list, set, tuple): filterValue
                query parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_aci_bindings, dict(
                filter_by=filter_by,
                filter_value=filter_value,
                page=page,
                size=size,
                sort=sort,
                sort_by=sort_by,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["nextPage", "href"],
            access_resource_list=["response"])

    def get_all_generator(self,
                          filter_by=None,
                          filter_value=None,
                          page=None,
                          size=None,
                          sort=None,
                          sort_by=None,
                          headers=None,
                          **query_parameters):
        """Alias for `get_aci_bindings_generator <#ciscoisesdk.
        api.v3_1_1.aci_bindings.
        AciBindings.get_aci_bindings_generator>`_
        """
        yield from get_next_page(
            self.get_aci_bindings, dict(
                filter_by=filter_by,
                filter_value=filter_value,
                page=page,
                size=size,
                sort=sort,
                sort_by=sort_by,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["nextPage", "href"],
            access_resource_list=["response"])

    def get_version(self,
                    headers=None,
                    **query_parameters):
        """This API helps to retrieve the version information related to
        the Cisco ACI bindings.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'Content-Type' in headers:
                check_type(headers.get('Content-Type'),
                           str, may_be_none=False)
            if 'Accept' in headers:
                check_type(headers.get('Accept'),
                           str, may_be_none=False)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/ers/config/acibindings/versioninfo')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d74b5214bad656c98f21e4968661c3c0_v3_1_1', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Classification rule API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *



from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class Classificationrule(object):
    """Identity Services Engine Classification rule API (version: 3.5.0).

    Wraps the Identity Services Engine Classification rule
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new Classificationrule
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(Classificationrule, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_classification_rule_list(self,
                                     headers=None,
                                     **query_parameters):
        """Classification rule  ISE uses ACI policy objects as conditions
        to determine which context goes to which Primary Sgt and
        Secondary Sgts.

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/trustsec/classification-policy')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_cbbd6a8b24250a6b4cd0224469a474f_v3_5_0', _api_response)

    def create_classification_rule_configuration(self,
                                                 mappings_count=None,
                                                 primary_sgt=None,
                                                 rule=None,
                                                 secondary_sgts=None,
                                                 headers=None,
                                                 payload=None,
                                                 active_validation=True,
                                                 **query_parameters):
        """Classification rule  Create Classification rule :     Filter
        configuration must include name and condition.
        Condition has hierarchical structure which define a set
        of conditions for which authoriztion policy rule could
        be match.     Condition can be either reference to a
        stored Library condition, using model
        ConditionReference    or dynamically built conditions
        which are not stored in the conditions Library, using
        models  ConditionAttributes, ConditionAndBlock,
        ConditionOrBlock .    .

        Args:
            mappings_count(integer): mappingsCount, property of the
                request body.
            primary_sgt(string): primarySgt, property of the request
                body.
            rule(object): Common attributes in rule
                authentication/authorization, property
                of the request body.
            secondary_sgts(list): Secondary Sgts used in the
                Classification Rule destination
                configurations, property of the request
                body (list of strings).
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'mappingsCount':
                    mappings_count,
                'primarySgt':
                    primary_sgt,
                'rule':
                    rule,
                'secondarySgts':
                    secondary_sgts,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_b3a8183dd45ea19a4857e14fe356ab_v3_5_0')\
                .validate(_payload)

        e_url = ('/api/v1/trustsec/classification-policy')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_b3a8183dd45ea19a4857e14fe356ab_v3_5_0', _api_response)

    def get_classification_rule_filter_preview(self,
                                               cdp_classification_rule=None,
                                               page_number=None,
                                               page_size=None,
                                               headers=None,
                                               payload=None,
                                               active_validation=True,
                                               **query_parameters):
        """Classification Policy Preview Classification Policy:     Filter
        configuration must include name and condition.
        Condition has hierarchical structure which define a set
        of conditions for which authoriztion policy rule could
        be match.     Condition can be either reference to a
        stored Library condition, using model
        ConditionReference    or dynamically built conditions
        which are not stored in the conditions Library, using
        models  ConditionAttributes, ConditionAndBlock,
        ConditionOrBlock .    .

        Args:
            page_number(integer): Page Number, property of the
                request body.
            page_size(integer): Page Size, property of the request
                body.
            cdp_classification_rule(object): Classification Rule
                details, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'PageNumber':
                    page_number,
                'PageSize':
                    page_size,
                'cdpClassificationRule':
                    cdp_classification_rule,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_f8a6116d2050518da89ba699dba15c5e_v3_5_0')\
                .validate(_payload)

        e_url = ('/api/v1/trustsec/classification-policy/preview')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_f8a6116d2050518da89ba699dba15c5e_v3_5_0', _api_response)

    def re_rank_classification_rules(self,
                                     classification_rule_rank_requests=None,
                                     headers=None,
                                     payload=None,
                                     active_validation=True,
                                     **query_parameters):
        """Classification rule  Re-rank Classification rules.     Provide
        list of existing Workload Classification Policies names,
        id, and newly desired rank    .

        Args:
            classification_rule_rank_requests(list):
                classificationRuleRankRequests, property
                of the request body (list of objects).
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'classificationRuleRankRequests':
                    classification_rule_rank_requests,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_c7f3352e2b5ba2f0467835c8d_v3_5_0')\
                .validate(_payload)

        e_url = ('/api/v1/trustsec/classification-policy/re-rank')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_c7f3352e2b5ba2f0467835c8d_v3_5_0', _api_response)

    def get_classification_rule_by_id(self,
                                      filter_id,
                                      headers=None,
                                      **query_parameters):
        """Classification rule  Get Classification rule  attributes.

        Args:
            filter_id(str): filterId path parameter. Inbound /
                Outbound Rule id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(filter_id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'filterId': filter_id,
        }

        e_url = ('/api/v1/trustsec/classification-policy/{filterId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ff95abf7721528693f4f2c8ba9c3f46_v3_5_0', _api_response)

    def update_classification_rule_by_id(self,
                                         filter_id,
                                         mappings_count=None,
                                         primary_sgt=None,
                                         rule=None,
                                         secondary_sgts=None,
                                         headers=None,
                                         payload=None,
                                         active_validation=True,
                                         **query_parameters):
        """Classification rule  Update Classification rule.

        Args:
            mappings_count(integer): mappingsCount, property of the
                request body.
            primary_sgt(string): primarySgt, property of the request
                body.
            rule(object): Common attributes in rule
                authentication/authorization, property
                of the request body.
            secondary_sgts(list): Secondary Sgts used in the
                Classification Rule destination
                configurations, property of the request
                body (list of strings).
            filter_id(str): filterId path parameter. Inbound /
                Outbound Rule id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)
        check_type(filter_id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'filterId': filter_id,
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'mappingsCount':
                    mappings_count,
                'primarySgt':
                    primary_sgt,
                'rule':
                    rule,
                'secondarySgts':
                    secondary_sgts,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_eb5bfbb2e9f75684b074f0cd62c5eeb9_v3_5_0')\
                .validate(_payload)

        e_url = ('/api/v1/trustsec/classification-policy/{filterId}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_eb5bfbb2e9f75684b074f0cd62c5eeb9_v3_5_0', _api_response)

    def delete_classification_rule_by_id(self,
                                         filter_id,
                                         headers=None,
                                         **query_parameters):
        """Classification rule  Delete Classification rule .

        Args:
            filter_id(str): filterId path parameter. Inbound /
                Outbound Rule id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(filter_id, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'filterId': filter_id,
        }

        e_url = ('/api/v1/trustsec/classification-policy/{filterId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a074d2116f05cb097bb032f0a32c2da_v3_5_0', _api_response)



# -*- coding: utf-8 -*-
"""Package helper functions and classes.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from builtins import *
from .exceptions import ApiError, RateLimitError
from .response_codes import RATE_LIMIT_RESPONSE_CODE


def check_response_code(response, expected_response_code, **kwargs):
    """Check response code against the expected code; raise ApiError.

    Checks the requests.response.status_code against the provided expected
    response code (erc), and raises a ApiError if they do not match.

    Args:
        response(requests.response): The response object returned by a request
            using the requests package.
        expected_response_code(int): The expected response code (HTTP response
            code).

    Raises:
        ApiError: If the requests.response.status_code does not match the
            provided expected response code (erc).

     """
    if response.status_code in expected_response_code:
        pass
    elif response.status_code == RATE_LIMIT_RESPONSE_CODE:
        raise RateLimitError(response, **kwargs)
    else:
        raise ApiError(response, **kwargs)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Trustsec Integration Rule - Dictionary Attributes API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *



from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class TrustsecIntegrationRuleDictionaryAttributes(object):
    """Identity Services Engine Trustsec Integration Rule - Dictionary Attributes API (version: 3.5.0).

    Wraps the Identity Services Engine Trustsec Integration Rule - Dictionary Attributes
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new TrustsecIntegrationRuleDictionaryAttributes
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(TrustsecIntegrationRuleDictionaryAttributes, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_common_policy_classification_dictionary_attributes_by_dictionary_name(self,
                                                                                  dictionary_name,
                                                                                  headers=None,
                                                                                  **query_parameters):
        """Trustsec Integraion Rule Returns list of dictionary attributes
        for Classification.

        Args:
            dictionary_name(str): dictionaryName path parameter. the
                name of the dictionary the dictionary
                attribute belongs to.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(dictionary_name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'dictionaryName': dictionary_name,
        }

        e_url = ('/api/v1/trustsec/classification-'
                 + 'policy/dictionaries/{dictionaryName}/attribute')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f938a08c4be5975b3319c25f2f749c3_v3_5_0', _api_response)

    def get_common_policy_classification_attributes_by_dictionary_name_by_attribute_name(self,
                                                                                         attribute_name,
                                                                                         dictionary_name,
                                                                                         headers=None,
                                                                                         **query_parameters):
        """Get a Dictionary Attribute.

        Args:
            attribute_name(str): attributeName path parameter. the
                dictionary attribute name.
            dictionary_name(str): dictionaryName path parameter. the
                name of the dictionary the dictionary
                attribute belongs to.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(attribute_name, str,
                   may_be_none=False)
        check_type(dictionary_name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'attributeName': attribute_name,
            'dictionaryName': dictionary_name,
        }

        e_url = ('/api/v1/trustsec/classification-policy/dictionaries/{dic'
                 + 'tionaryName}/attribute/{attributeName}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f0794e5915b4292abc09bec0dcdd0_v3_5_0', _api_response)

    def get_common_policy_ingress_dictionary_attributes_by_dictionary_name(self,
                                                                           dictionary_name,
                                                                           headers=None,
                                                                           **query_parameters):
        """Trustsec Integraion Rule Returns list of dictionary attributes
        for Inbound.

        Args:
            dictionary_name(str): dictionaryName path parameter. the
                name of the dictionary the dictionary
                attribute belongs to.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(dictionary_name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'dictionaryName': dictionary_name,
        }

        e_url = ('/api/v1/trustsec/inbound-'
                 + 'rule/dictionaries/{dictionaryName}/attribute')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f7a68260b928570b959ad0780acd181a_v3_5_0', _api_response)

    def get_common_policy_ingress_attributes_by_dictionary_name_by_attribute_name(self,
                                                                                  attribute_name,
                                                                                  dictionary_name,
                                                                                  headers=None,
                                                                                  **query_parameters):
        """Get a Dictionary Attribute.

        Args:
            attribute_name(str): attributeName path parameter. the
                dictionary attribute name.
            dictionary_name(str): dictionaryName path parameter. the
                name of the dictionary the dictionary
                attribute belongs to.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(attribute_name, str,
                   may_be_none=False)
        check_type(dictionary_name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'attributeName': attribute_name,
            'dictionaryName': dictionary_name,
        }

        e_url = ('/api/v1/trustsec/inbound-rule/dictionaries/{dictionaryNa'
                 + 'me}/attribute/{attributeName}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_fd7d76dbe07a59a0883dc85d75fee7d5_v3_5_0', _api_response)

    def get_common_policy_egress_dictionary_attributes_by_dictionary_name(self,
                                                                          dictionary_name,
                                                                          headers=None,
                                                                          **query_parameters):
        """Trustsec Integraion Rule Returns list of dictionary attributes
        for outbound.

        Args:
            dictionary_name(str): dictionaryName path parameter. the
                name of the dictionary the dictionary
                attribute belongs to.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(dictionary_name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'dictionaryName': dictionary_name,
        }

        e_url = ('/api/v1/trustsec/outbound-'
                 + 'rule/dictionaries/{dictionaryName}/attribute')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_aa3278f5051c4aab573d7367ebd73_v3_5_0', _api_response)

    def get_common_policy_egress_attributes_by_dictionary_name_by_attribute_name(self,
                                                                                 attribute_name,
                                                                                 dictionary_name,
                                                                                 headers=None,
                                                                                 **query_parameters):
        """Get a Dictionary Attribute.

        Args:
            attribute_name(str): attributeName path parameter. the
                dictionary attribute name.
            dictionary_name(str): dictionaryName path parameter. the
                name of the dictionary the dictionary
                attribute belongs to.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            if 'X-Request-ID' in headers:
                check_type(headers.get('X-Request-ID'),
                           str)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(attribute_name, str,
                   may_be_none=False)
        check_type(dictionary_name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'attributeName': attribute_name,
            'dictionaryName': dictionary_name,
        }

        e_url = ('/api/v1/trustsec/outbound-rule/dictionaries/{dictionaryN'
                 + 'ame}/attribute/{attributeName}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b64207755bd9ec36c74ef795317_v3_5_0', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Menu Access API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *



from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class MenuAccess(object):
    """Identity Services Engine Menu Access API (version: 3.5.0).

    Wraps the Identity Services Engine Menu Access
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new MenuAccess
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(MenuAccess, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def list_all_menu_access(self,
                             filter=None,
                             filter_type=None,
                             page=None,
                             size=None,
                             sort=None,
                             sort_by=None,
                             headers=None,
                             **query_parameters):
        """Get all the menu access created in Cisco ISE, example super
        admin menu access,Policy admin menu Access.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            sort(str): sort query parameter. sort type asc or desc.
            sort_by(str): sortBy query parameter. sort column by
                which objects needs to be sorted.
            filter(str): filter query parameter.        Simple
                filtering  should be available through
                the filter query string parameter. The
                structure of a filter is a triplet of
                field operator and value separated with
                dots. More than one filter can be sent.
                The logical operator common to ALL
                filter criteria will be by default AND,
                and can be changed by using the
                "filterType=or"  query string parameter.
                Each resource Data model description
                should specify if an attribute is a
                filtered field.            OPERATOR
                DESCRIPTION           EQ   Equals
                NEQ   Not Equals       GT   Greater Than
                LT   Less Then       STARTSW   Starts
                With       NSTARTSW   Not Starts With
                ENDSW   Ends With       NENDSW   Not
                Ends With       CONTAINS   Contains
                NCONTAINS   Not Contains         .
            filter_type(str): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))
        check_type(sort, str)
        check_type(sort_by, str)
        check_type(filter, str)
        check_type(filter_type, str)

        _params = {
            'page':
                page,
            'size':
                size,
            'sort':
                sort,
            'sortBy':
                sort_by,
            'filter':
                filter,
            'filterType':
                filter_type,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/rbac/menu-access')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a5a65aa6ac41185099dfcc2d_v3_5_0', _api_response)

    def list_all_menu_access_generator(self,
                                       filter=None,
                                       filter_type=None,
                                       page=None,
                                       size=None,
                                       sort=None,
                                       sort_by=None,
                                       headers=None,
                                       **query_parameters):
        """Get all the menu access created in Cisco ISE, example super
        admin menu access,Policy admin menu Access.

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            sort(str): sort query parameter. sort type asc or desc.
            sort_by(str): sortBy query parameter. sort column by
                which objects needs to be sorted.
            filter(str): filter query parameter.        Simple
                filtering  should be available through
                the filter query string parameter. The
                structure of a filter is a triplet of
                field operator and value separated with
                dots. More than one filter can be sent.
                The logical operator common to ALL
                filter criteria will be by default AND,
                and can be changed by using the
                "filterType=or"  query string parameter.
                Each resource Data model description
                should specify if an attribute is a
                filtered field.            OPERATOR
                DESCRIPTION           EQ   Equals
                NEQ   Not Equals       GT   Greater Than
                LT   Less Then       STARTSW   Starts
                With       NSTARTSW   Not Starts With
                ENDSW   Ends With       NENDSW   Not
                Ends With       CONTAINS   Contains
                NCONTAINS   Not Contains         .
            filter_type(str): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.list_all_menu_access, dict(
                filter=filter,
                filter_type=filter_type,
                page=page,
                size=size,
                sort=sort,
                sort_by=sort_by,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["nextPage", "href"],
            access_resource_list=["response"])

    def update_menu_access(self,
                           description=None,
                           menu_access_privileges=None,
                           name=None,
                           headers=None,
                           payload=None,
                           active_validation=True,
                           **query_parameters):
        """Update details of menu access present in Cisco ISE based on the
        permissions required for specific tabs.

        Args:
            description(string): A brief description of the menu
                access request., property of the request
                body.
            menu_access_privileges(list): menuAccessPrivileges,
                property of the request body (list of
                objects).
            name(string): The name of the menu access request.,
                property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'description':
                    description,
                'menuAccessPrivileges':
                    menu_access_privileges,
                'name':
                    name,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_df8ee11dc4a85b46ad2fe44eb3a89475_v3_5_0')\
                .validate(_payload)

        e_url = ('/api/v1/rbac/menu-access')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_df8ee11dc4a85b46ad2fe44eb3a89475_v3_5_0', _api_response)

    def create_menu_access(self,
                           description=None,
                           menu_access_privileges=None,
                           name=None,
                           headers=None,
                           payload=None,
                           active_validation=True,
                           **query_parameters):
        """Create  menu access in Cisco ISE based on the access permissions
        required to the tabs.

        Args:
            description(string): A brief description of the menu
                access request., property of the request
                body.
            menu_access_privileges(list): menuAccessPrivileges,
                property of the request body (list of
                objects).
            name(string): The name of the menu access request.,
                property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'description':
                    description,
                'menuAccessPrivileges':
                    menu_access_privileges,
                'name':
                    name,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_c0c869427aa5cd9bfdbf2a1258a7179_v3_5_0')\
                .validate(_payload)

        e_url = ('/api/v1/rbac/menu-access')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_c0c869427aa5cd9bfdbf2a1258a7179_v3_5_0', _api_response)

    def get_menu_access_by_name(self,
                                name,
                                headers=None,
                                **query_parameters):
        """Get details of requested menu access available in Cisco ISE,
        which has the details of all the accessible tabs.

        Args:
            name(str): name path parameter. The name of the menu
                access.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }

        e_url = ('/api/v1/rbac/menu-access/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_e78f56baa572caf543e9172a6654a_v3_5_0', _api_response)

    def delete_menu_access(self,
                           name,
                           headers=None,
                           **query_parameters):
        """Delete menu access API in Cisco ISE is used to delete menu
        access based on the role fulfilled.

        Args:
            name(str): name path parameter. The name of the menu
                access.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(name, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'name': name,
        }

        e_url = ('/api/v1/rbac/menu-access/{name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_ac5a3d0038cc52548f6fb11df4bfa006_v3_5_0', _api_response)



# -*- coding: utf-8 -*-
"""Cisco Identity Services Engine Native IPsec API wrapper.

Copyright (c) 2021 Cisco and/or its affiliates.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from builtins import *
from ...pagination import get_next_page
from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class NativeIpsec(object):
    """Identity Services Engine Native IPsec API (version: 3.3_patch_1).

    Wraps the Identity Services Engine Native IPsec
    API and exposes the API as native Python
    methods that return native Python objects.


    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new NativeIpsec
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the Identity Services Engine service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(NativeIpsec, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def get_ipsec_enabled_nodes(self,
                                filter=None,
                                filter_type=None,
                                page=None,
                                size=None,
                                sort=None,
                                sort_by=None,
                                headers=None,
                                **query_parameters):
        """ Returns all the IPsec enabled nodes with configuration details.
        This API supports filtering, sorting and pagination.
        The attributes that are suppported for filtering are:
        hostName   nadIp   status   authType     The attribute
        that is suppported for sorting is:      hostName    .

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            filter(str): filter query parameter.
                Simple filtering  should be available
                through the filter query string
                parameter. The structure of a filter is
                a triplet of field operator and value
                separated with dots. More than one
                filter can be sent. The logical operator
                common to ALL filter criteria will be by
                default AND, and can be changed by using
                the  "filterType=or"  query string
                parameter. Each resource Data model
                description should specify if an
                attribute is a filtered field.
                OPERATOR   DESCRIPTION   APPLICABLE ON
                FIELDS           EQ   Equals   authType
                NEQ   Not Equals   authType       EQ
                Equals   hostName       NEQ   Not Equals
                hostName       EQ   Equals   nadIp
                NEQ   Not Equals   nadIp       EQ
                Equals   status       NEQ   Not Equals
                status         .
            filter_type(str): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            sort(str): sort query parameter. sort type asc or
                desc.
            sort_by(str): sortBy query parameter. Sort column
                The IPsec enabled nodes are sorted based
                on the columns. This is applicable for
                the field hostName.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(page, (int, str, list))
        check_type(size, (int, str, list))
        check_type(filter, str)
        check_type(filter_type, str)
        check_type(sort, str)
        check_type(sort_by, str)

        _params = {
            'page':
                page,
            'size':
                size,
            'filter':
                filter,
            'filterType':
                filter_type,
            'sort':
                sort,
            'sortBy':
                sort_by,
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/ipsec')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_d9fe5cd54278a69cb1b5c800c32_v3_3_patch_1', _api_response)

    def get_ipsec_enabled_nodes_generator(self,
                                          filter=None,
                                          filter_type=None,
                                          page=None,
                                          size=None,
                                          sort=None,
                                          sort_by=None,
                                          headers=None,
                                          **query_parameters):
        """ Returns all the IPsec enabled nodes with configuration details.
        This API supports filtering, sorting and pagination.
        The attributes that are suppported for filtering are:
        hostName   nadIp   status   authType     The attribute
        that is suppported for sorting is:      hostName    .

        Args:
            page(int): page query parameter. Page number.
            size(int): size query parameter. Number of objects
                returned per page.
            filter(str): filter query parameter.
                Simple filtering  should be available
                through the filter query string
                parameter. The structure of a filter is
                a triplet of field operator and value
                separated with dots. More than one
                filter can be sent. The logical operator
                common to ALL filter criteria will be by
                default AND, and can be changed by using
                the  "filterType=or"  query string
                parameter. Each resource Data model
                description should specify if an
                attribute is a filtered field.
                OPERATOR   DESCRIPTION   APPLICABLE ON
                FIELDS           EQ   Equals   authType
                NEQ   Not Equals   authType       EQ
                Equals   hostName       NEQ   Not Equals
                hostName       EQ   Equals   nadIp
                NEQ   Not Equals   nadIp       EQ
                Equals   status       NEQ   Not Equals
                status         .
            filter_type(str): filterType query parameter. The
                logical operator common to ALL filter
                criteria will be by default AND, and can
                be changed by using the parameter.
            sort(str): sort query parameter. sort type asc or
                desc.
            sort_by(str): sortBy query parameter. Sort column
                The IPsec enabled nodes are sorted based
                on the columns. This is applicable for
                the field hostName.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:
            Generator: A generator object containing the following object.

              + RestResponse: REST response with following properties:

                  - headers(MyDict): response headers.
                  - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                    or the bracket notation.
                  - content(bytes): representation of the request's response
                  - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """

        yield from get_next_page(
            self.get_ipsec_enabled_nodes, dict(
                filter=filter,
                filter_type=filter_type,
                page=page,
                size=size,
                sort=sort,
                sort_by=sort_by,
                headers=headers,
                **query_parameters
            ),
            access_next_list=["nextPage", "href"],
            access_resource_list=["response"])

    def update_ipsec_connection_config(self,
                                       auth_type=None,
                                       cert_id=None,
                                       configure_vti=None,
                                       esp_ah_protocol=None,
                                       host_name=None,
                                       iface=None,
                                       ike_re_auth_time=None,
                                       ike_version=None,
                                       local_internal_ip=None,
                                       mode_option=None,
                                       nad_ip=None,
                                       phase_one_dhgroup=None,
                                       phase_one_encryption_algo=None,
                                       phase_one_hash_algo=None,
                                       phase_one_life_time=None,
                                       phase_two_dhgroup=None,
                                       phase_two_encryption_algo=None,
                                       phase_two_hash_algo=None,
                                       phase_two_life_time=None,
                                       psk=None,
                                       remote_peer_internal_ip=None,
                                       headers=None,
                                       payload=None,
                                       active_validation=True,
                                       **query_parameters):
        """  Updates the configuration of existing IPsec connection.  The
        following parameters are present in the PUT request
        body:         PARAMETER   DESCRIPTION   EXAMPLE
        id *required   ID of the existing IPsec configuration.
        "id": "7c9484cf-0ebc-47ad-a9ef-bc12729ed73b"       iface
        *required   Ethernet port used for establishing
        connection   "iface": "0"       psk *required   Pre-
        shared key used for establishing connection.   "psk":
        "psk12345"       authType *required   Pre-shared key
        used for establishing connection.   "authType": "psk"
        configureVti   Used For VTI Configurations
        "configureVti": "false"       remotePeerInternalIp   VTI
        Internal IP of the NAD   "remotePeerInternalIp":
        "1.2.3.1"       localInternalIp   IP address assigned to
        the VTI interface so this would be the internal ip
        "localInternalIp": "1.1.3.1"       certId *required   ID
        of the certificate for establishing connection.
        "certId": "21323243545433"       phaseOneEncryptionAlgo
        *required   Phase-one encryption algorithm used for
        establishing connection.   "phaseOneEncryptionAlgo":
        "aes"       phaseTwoEncryptionAlgo *required   Phase-two
        encryption algorithm used for establishing connection.
        "phaseTwoEncryptionAlgo": "aes"       espAhProtocol
        *required   Encryption protocol used for establishing
        connection.   "espAhProtocol": "ah"
        phaseOneHashAlgo *required   Phase-one hashing algorithm
        used for establishing connection.   "phaseOneHashAlgo":
        "sha"       phaseTwoHashAlgo *required   Phase-two
        hashing algorithm used for establishing connection.
        "phaseTwoHashAlgo": "sha"       phaseOneDHGroup
        *required   Phase-one DH group used for establishing
        connection.   "phaseOneDHGroup": "GROUP1"
        phaseTwoDHGroup   Phase-two DH group used for
        establishing connection.   "phaseTwoDHGroup": "GROUP1"
        phaseOneLifeTime   DH Phase-one connection lifetime.
        "phaseOneLifeTime": 14400       phaseTwoLifeTime   DH
        Phase-two connection lifetime.   "phaseTwoLifeTime":
        14400       ikeVersion *required   IKE version.
        "ikeVersion": "1"       ikeReAuthTime   IKE re-
        authentication time.   "ikeReAuthTime": 86400
        nadIp *required   NAD IP for establishing connection.
        "nadIp": "1.1.1.1"       modeOption *required   The Mode
        type used for establishing the connection.
        "modeOption": "tunnel"         NOTE:    psk  field is
        mandatory if authType=psk   certId  field is mandatory
        if authType=x509    If FIPS mode is on.:      Cannot
        choose DES or 3DES for Phase-one and Phase-two
        Encryption algorithms.   PSK length must be 14
        characters or more.   DH Groups 1, 2, and 5 cannot be
        chosen for Phase-one and Phase-two fields.      .

        Args:
            auth_type(string): Authentication type for establishing
                connection, property of the request
                body. Available values are 'psk' and
                'x509'.
            cert_id(string): ID of the certificate for establishing
                connection, property of the request
                body.
            configure_vti(boolean): Authentication type for
                establishing connection, property of the
                request body.
            esp_ah_protocol(string): Encryption protocol used for
                establishing connection, property of the
                request body. Available values are 'ah'
                and 'esp'.
            host_name(string): Hostname of the node, property of the
                request body.
            iface(string): Ethernet port of the node, property of
                the request body.
            ike_re_auth_time(integer): IKE re-authentication time,
                property of the request body.
            ike_version(string): IKE version, property of the
                request body. Available values are '1'
                and '2'.
            local_internal_ip(string): Local Tunnel IP address,
                property of the request body.
            mode_option(string): The Mode type used for establishing
                the connection, property of the request
                body. Available values are 'transport'
                and 'tunnel'.
            nad_ip(string): NAD IP address for establishing
                connection, property of the request
                body.
            phase_one_dhgroup(string): Phase-one DH group used for
                establishing connection, property of the
                request body. Available values are
                'GROUP1', 'GROUP14', 'GROUP15',
                'GROUP16', 'GROUP19', 'GROUP2',
                'GROUP20', 'GROUP21', 'GROUP24' and
                'GROUP5'.
            phase_one_encryption_algo(string): Phase-one encryption
                algorithm used for establishing
                connection, property of the request
                body. Available values are '3des',
                'aes', 'aes128', 'aes192', 'aes256' and
                'des'.
            phase_one_hash_algo(string): Phase-one hashing algorithm
                used for establishing connection,
                property of the request body. Available
                values are 'sha', 'sha256', 'sha384' and
                'sha512'.
            phase_one_life_time(integer): Phase-one connection
                lifetime, property of the request body.
            phase_two_dhgroup(string): Phase-two DH group used for
                establishing connection, property of the
                request body. Available values are
                'GROUP1', 'GROUP14', 'GROUP15',
                'GROUP16', 'GROUP19', 'GROUP2',
                'GROUP20', 'GROUP21', 'GROUP24',
                'GROUP5' and 'NONE'.
            phase_two_encryption_algo(string): Phase-two encryption
                algorithm used for establishing
                connection, property of the request
                body. Available values are '3des',
                'aes', 'aes128', 'aes192', 'aes256',
                'des', 'gcm' and 'gmac'.
            phase_two_hash_algo(string): Phase-two hashing algorithm
                used for establishing connection,
                property of the request body. Available
                values are 'sha', 'sha256', 'sha384' and
                'sha512'.
            phase_two_life_time(integer): Phase-two connection
                lifetime, property of the request body.
            psk(string): Pre-shared key used for establishing
                connection, property of the request
                body.
            remote_peer_internal_ip(string): Remote Tunnel IP
                address, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'authType':
                    auth_type,
                'certId':
                    cert_id,
                'configureVti':
                    configure_vti,
                'espAhProtocol':
                    esp_ah_protocol,
                'hostName':
                    host_name,
                'iface':
                    iface,
                'ikeReAuthTime':
                    ike_re_auth_time,
                'ikeVersion':
                    ike_version,
                'localInternalIp':
                    local_internal_ip,
                'modeOption':
                    mode_option,
                'nadIp':
                    nad_ip,
                'phaseOneDHGroup':
                    phase_one_dhgroup,
                'phaseOneEncryptionAlgo':
                    phase_one_encryption_algo,
                'phaseOneHashAlgo':
                    phase_one_hash_algo,
                'phaseOneLifeTime':
                    phase_one_life_time,
                'phaseTwoDHGroup':
                    phase_two_dhgroup,
                'phaseTwoEncryptionAlgo':
                    phase_two_encryption_algo,
                'phaseTwoHashAlgo':
                    phase_two_hash_algo,
                'phaseTwoLifeTime':
                    phase_two_life_time,
                'psk':
                    psk,
                'remotePeerInternalIp':
                    remote_peer_internal_ip,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_d4fb0706d0f5226b8691538a2bc7309_v3_3_patch_1')\
                .validate(_payload)

        e_url = ('/api/v1/ipsec')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers,
                                              **request_params)

        else:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              **request_params)

        return self._object_factory('bpm_d4fb0706d0f5226b8691538a2bc7309_v3_3_patch_1', _api_response)

    def create_ipsec_connection(self,
                                auth_type=None,
                                cert_id=None,
                                configure_vti=None,
                                esp_ah_protocol=None,
                                host_name=None,
                                iface=None,
                                ike_re_auth_time=None,
                                ike_version=None,
                                local_internal_ip=None,
                                mode_option=None,
                                nad_ip=None,
                                phase_one_dhgroup=None,
                                phase_one_encryption_algo=None,
                                phase_one_hash_algo=None,
                                phase_one_life_time=None,
                                phase_two_dhgroup=None,
                                phase_two_encryption_algo=None,
                                phase_two_hash_algo=None,
                                phase_two_life_time=None,
                                psk=None,
                                remote_peer_internal_ip=None,
                                headers=None,
                                payload=None,
                                active_validation=True,
                                **query_parameters):
        """  Creates an IPsec connection.  The following parameters are
        present in the POST request body:         PARAMETER
        DESCRIPTION   EXAMPLE           hostName *required
        Hostname of the node for which IPsec should be enabled
        "hostName": "ise-host1"       iface *required   Ethernet
        port used for establishing connection   "iface": "0"
        psk *required   Pre-shared key used for establishing
        connection.   "psk": "psk12345"       authType *required
        Pre-shared key used for establishing connection.
        "authType": "psk"       configureVti   Used For VTI
        Configurations   "configureVti": "false"
        remotePeerInternalIp   VTI Internal IP of the NAD
        "remotePeerInternalIp": "1.2.3.1"       localInternalIp
        IP address assigned to the VTI interface so this would
        be the internal ip   "localInternalIp": "1.1.3.1"
        certId *required   ID of the certificate for
        establishing connection.   "certId": "21323243545433"
        phaseOneEncryptionAlgo *required   Phase-one encryption
        algorithm used for establishing connection.
        "phaseOneEncryptionAlgo": "aes"
        phaseTwoEncryptionAlgo *required   Phase-two encryption
        algorithm used for establishing connection.
        "phaseTwoEncryptionAlgo": "aes"       espAhProtocol
        *required   Encryption protocol used for establishing
        connection.   "espAhProtocol": "ah"
        phaseOneHashAlgo *required   Phase-one hashing algorithm
        used for establishing connection.   "phaseOneHashAlgo":
        "sha"       phaseTwoHashAlgo *required   Phase-two
        hashing algorithm used for establishing connection.
        "phaseTwoHashAlgo": "sha"       phaseOneDHGroup
        *required   Phase-one DH group used for establishing
        connection.   "phaseOneDHGroup": "GROUP1"
        phaseTwoDHGroup   Phase-two DH group used for
        establishing connection.   "phaseTwoDHGroup": "GROUP1"
        phaseOneLifeTime   DH Phase-one connection lifetime.
        "phaseOneLifeTime": 14400       phaseTwoLifeTime   DH
        Phase-two connection lifetime.   "phaseTwoLifeTime":
        14400       ikeVersion *required   IKE version.
        "ikeVersion": "1"       ikeReAuthTime   IKE re-
        authentication time.   "ikeReAuthTime": 86400
        nadIp *required   NAD IP for establishing the
        connection.   "nadIp": "1.1.1.1"       modeOption
        *required   The Mode type used for establishing the
        connection.   "modeOption": "tunnel"         NOTE:
        psk  field is mandatory if authType=psk   certId  field
        is mandatory if authType=x509    If FIPS mode is on.:
        Cannot choose DES or 3DES for Phase-one and Phase-two
        Encryption algorithms.   PSK length must be 14
        characters or more.   DH Groups 1, 2, and 5 cannot be
        chosen for Phase-one and Phase-two fields.      .

        Args:
            auth_type(string): Authentication type for establishing
                connection, property of the request
                body. Available values are 'psk' and
                'x509'.
            cert_id(string): ID of the certificate for establishing
                connection, property of the request
                body.
            configure_vti(boolean): Authentication type for
                establishing connection, property of the
                request body.
            esp_ah_protocol(string): Encryption protocol used for
                establishing connection, property of the
                request body. Available values are 'ah'
                and 'esp'.
            host_name(string): Hostname of the node, property of the
                request body.
            iface(string): Ethernet port of the node, property of
                the request body.
            ike_re_auth_time(integer): IKE re-authentication time,
                property of the request body.
            ike_version(string): IKE version, property of the
                request body. Available values are '1'
                and '2'.
            local_internal_ip(string): Local Tunnel IP address,
                property of the request body.
            mode_option(string): The Mode type used for establishing
                the connection, property of the request
                body. Available values are 'transport'
                and 'tunnel'.
            nad_ip(string): NAD IP address for establishing
                connection, property of the request
                body.
            phase_one_dhgroup(string): Phase-one DH group used for
                establishing connection, property of the
                request body. Available values are
                'GROUP1', 'GROUP14', 'GROUP15',
                'GROUP16', 'GROUP19', 'GROUP2',
                'GROUP20', 'GROUP21', 'GROUP24' and
                'GROUP5'.
            phase_one_encryption_algo(string): Phase-one encryption
                algorithm used for establishing
                connection, property of the request
                body. Available values are '3des',
                'aes', 'aes128', 'aes192', 'aes256' and
                'des'.
            phase_one_hash_algo(string): Phase-one hashing algorithm
                used for establishing connection,
                property of the request body. Available
                values are 'sha', 'sha256', 'sha384' and
                'sha512'.
            phase_one_life_time(integer): Phase-one connection
                lifetime, property of the request body.
            phase_two_dhgroup(string): Phase-two DH group used for
                establishing connection, property of the
                request body. Available values are
                'GROUP1', 'GROUP14', 'GROUP15',
                'GROUP16', 'GROUP19', 'GROUP2',
                'GROUP20', 'GROUP21', 'GROUP24',
                'GROUP5' and 'NONE'.
            phase_two_encryption_algo(string): Phase-two encryption
                algorithm used for establishing
                connection, property of the request
                body. Available values are '3des',
                'aes', 'aes128', 'aes192', 'aes256',
                'des', 'gcm' and 'gmac'.
            phase_two_hash_algo(string): Phase-two hashing algorithm
                used for establishing connection,
                property of the request body. Available
                values are 'sha', 'sha256', 'sha384' and
                'sha512'.
            phase_two_life_time(integer): Phase-two connection
                lifetime, property of the request body.
            psk(string): Pre-shared key used for establishing
                connection, property of the request
                body.
            remote_peer_internal_ip(string): Remote Tunnel IP
                address, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'authType':
                    auth_type,
                'certId':
                    cert_id,
                'configureVti':
                    configure_vti,
                'espAhProtocol':
                    esp_ah_protocol,
                'hostName':
                    host_name,
                'iface':
                    iface,
                'ikeReAuthTime':
                    ike_re_auth_time,
                'ikeVersion':
                    ike_version,
                'localInternalIp':
                    local_internal_ip,
                'modeOption':
                    mode_option,
                'nadIp':
                    nad_ip,
                'phaseOneDHGroup':
                    phase_one_dhgroup,
                'phaseOneEncryptionAlgo':
                    phase_one_encryption_algo,
                'phaseOneHashAlgo':
                    phase_one_hash_algo,
                'phaseOneLifeTime':
                    phase_one_life_time,
                'phaseTwoDHGroup':
                    phase_two_dhgroup,
                'phaseTwoEncryptionAlgo':
                    phase_two_encryption_algo,
                'phaseTwoHashAlgo':
                    phase_two_hash_algo,
                'phaseTwoLifeTime':
                    phase_two_life_time,
                'psk':
                    psk,
                'remotePeerInternalIp':
                    remote_peer_internal_ip,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_ef20d5d63551dbe25878007747598_v3_3_patch_1')\
                .validate(_payload)

        e_url = ('/api/v1/ipsec')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_ef20d5d63551dbe25878007747598_v3_3_patch_1', _api_response)

    def bulk_ip_sec_operation(self,
                              item_list=None,
                              operation=None,
                              headers=None,
                              payload=None,
                              active_validation=True,
                              **query_parameters):
        """Create, update, disable, enable and remove IPsec connections in
        bulk.

        Args:
            item_list(list): ItemList, property of the request body
                (list of objects).
            operation(string): operation, property of the request
                body. Available values are 'Create',
                'Delete', 'Disable', 'Enable' and
                'Update'.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        is_xml_payload = 'application/xml' in _headers.get('Content-Type', [])
        if active_validation and is_xml_payload:
            check_type(payload, str)
        if active_validation and not is_xml_payload:
            check_type(payload, dict)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }
        if is_xml_payload:
            _payload = payload
        else:
            _payload = {
                'ItemList':
                    item_list,
                'operation':
                    operation,
            }
            _payload.update(payload or {})
            _payload = dict_from_items_with_values(_payload)
        if active_validation and not is_xml_payload:
            self._request_validator('jsd_eca8a69196d555a1b7e71cf190bd2641_v3_3_patch_1')\
                .validate(_payload)

        e_url = ('/api/v1/ipsec/bulk')
        endpoint_full_url = apply_path_params(e_url, path_params)

        request_params = {'data': _payload} if is_xml_payload else {'json': _payload}
        if with_custom_headers:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               headers=_headers,
                                               **request_params)
        else:
            _api_response = self._session.post(endpoint_full_url, params=_params,
                                               **request_params)

        return self._object_factory('bpm_eca8a69196d555a1b7e71cf190bd2641_v3_3_patch_1', _api_response)

    def get_ip_sec_certificates(self,
                                headers=None,
                                **query_parameters):
        """ Returns all the certificates for IPsec role.   .

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        e_url = ('/api/v1/ipsec/certificates')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a5e87f906e357bcb972c60ee082d89b_v3_3_patch_1', _api_response)

    def disable_ipsec_connection(self,
                                 host_name,
                                 nad_ip,
                                 headers=None,
                                 **query_parameters):
        """Disables an enabled IPsec node connection.

        Args:
            host_name(str): hostName path parameter. Hostname
                of the deployed node.
            nad_ip(str): nadIp path parameter. IP address of
                the NAD.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(host_name, str,
                   may_be_none=False)
        check_type(nad_ip, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'hostName': host_name,
            'nadIp': nad_ip,
        }

        e_url = ('/api/v1/ipsec/disable/{hostName}/{nadIp}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.put(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f159a76b6aa52c2b0d0a2fe31dcae91_v3_3_patch_1', _api_response)

    def enable_ipsec_connection(self,
                                host_name,
                                nad_ip,
                                headers=None,
                                **query_parameters):
        """Enables an disabled IPsec node connection.

        Args:
            host_name(str): hostName path parameter. Hostname
                of the deployed node.
            nad_ip(str): nadIp path parameter. IP address of
                the NAD.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(host_name, str,
                   may_be_none=False)
        check_type(nad_ip, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'hostName': host_name,
            'nadIp': nad_ip,
        }

        e_url = ('/api/v1/ipsec/enable/{hostName}/{nadIp}')
        endpoint_full_url = apply_path_params(e_url, path_params)

        if with_custom_headers:
            _api_response = self._session.put(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.put(endpoint_full_url, params=_params)

        return self._object_factory('bpm_b57a1b126cb53ed8533eb22db13719d_v3_3_patch_1', _api_response)

    def get_ipsec_node(self,
                       host_name,
                       nad_ip,
                       headers=None,
                       **query_parameters):
        """Returns the IPsec configuration details of a given node with the
        hostname and the NAD IP.

        Args:
            host_name(str): hostName path parameter. Hostname
                of the deployed node.
            nad_ip(str): nadIp path parameter. IP address of
                the NAD.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(host_name, str,
                   may_be_none=False)
        check_type(nad_ip, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'hostName': host_name,
            'nadIp': nad_ip,
        }

        e_url = ('/api/v1/ipsec/{hostName}/{nadIp}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.get(endpoint_full_url, params=_params,
                                              headers=_headers)
        else:
            _api_response = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_a5dd6bd055c73bc02c8a389fd353f_v3_3_patch_1', _api_response)

    def remove_ipsec_connection(self,
                                host_name,
                                nad_ip,
                                headers=None,
                                **query_parameters):
        """Removes an enabled IPsec node connection.

        Args:
            host_name(str): hostName path parameter. Hostname
                of the deployed node.
            nad_ip(str): nadIp path parameter. IP address of
                the NAD.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **query_parameters: Additional query parameters (provides
                support for parameters that may be added in the future).

        Returns:

            RestResponse: REST response with following properties:

              - headers(MyDict): response headers.
              - response(MyDict): response body as a MyDict object. Access the object's properties by using the dot notation
                or the bracket notation.
              - content(bytes): representation of the request's response
              - text(str): representation of the request's response

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the Identity Services Engine cloud returns an error.
        """
        check_type(headers, dict)

        if headers is not None:
            pass

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True
        check_type(host_name, str,
                   may_be_none=False)
        check_type(nad_ip, str,
                   may_be_none=False)

        _params = {
        }
        _params.update(query_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'hostName': host_name,
            'nadIp': nad_ip,
        }

        e_url = ('/api/v1/ipsec/{hostName}/{nadIp}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            _api_response = self._session.delete(endpoint_full_url, params=_params,
                                                 headers=_headers)
        else:
            _api_response = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_f2a9c1f22035982beaa10f027814d01_v3_3_patch_1', _api_response)
